#!/usr/bin/perl -wT
#
# ==========================================================================
#
# ZoneMinder Trendnet TV-IP400W Camera Control Script, $Date: 2006/05/08 12:37:48 $, $Revision: 1.5 $
# Copyright (C) 2007  Peter Hulst
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ==========================================================================
#
# This script continuously monitors the recorded events for the given
# monitor and applies any filters which would delete and/or upload 
# matching events
#
use strict;

# ==========================================================================
#
# These are the elements you can edit to suit your installation
#
# ==========================================================================

use constant DBG_ID => "zmctrl-trendnet"; # Tag that appears in debug to identify source
use constant DBG_LEVEL => 0; # 0 is errors, warnings and info only, > 0 for debug

# ==========================================================================

use ZoneMinder;
use Getopt::Long;

use constant PANTILT => "PANTILTCONTROL.CGI";

$| = 1;

$ENV{PATH}  = '/bin:/usr/bin';
$ENV{SHELL} = '/bin/sh' if exists $ENV{SHELL};
delete @ENV{qw(IFS CDPATH ENV BASH_ENV)};

sub Usage
{
	print( "Usage: zmcontrol-trendnet-ip400.pl <various options>");
	exit( -1 );
}

zmDbgInit( DBG_ID, level=>DBG_LEVEL );

my $arg_string = join( " ", @ARGV );

my $address;
my $command;
my ( $speed, $step );
my ( $xcoord, $ycoord );
my ( $width, $height );
my ( $panspeed, $tiltspeed );
my ( $panstep, $tiltstep );
my $preset;

if ( !GetOptions(
	'address=s'=>\$address,
	'command=s'=>\$command,
	'speed=i'=>\$speed,
	'step=i'=>\$step,
	'xcoord=i'=>\$xcoord,
	'ycoord=i'=>\$ycoord,
	'width=i'=>\$width,
	'height=i'=>\$height,
	'panspeed=i'=>\$panspeed,
	'tiltspeed=i'=>\$tiltspeed,
	'panstep=i'=>\$panstep,
	'tiltstep=i'=>\$tiltstep,
	'preset=i'=>\$preset
	)
)
{
	Usage();
}

if ( !$address )
{
	Usage();
}

Debug( $arg_string."\n" );

srand( time() );

sub printMsg
{
	my $msg = shift;
	my $msg_len = length($msg);

	Debug( $msg."[".$msg_len."]\n" );
}

sub sendCmd
{
        my $cmd = shift;
	my $parms = shift;

	my $result = undef;

	printMsg( $cmd, "Tx" );

	use LWP::UserAgent;
	my $ua = LWP::UserAgent->new;
	$ua->agent( "ZoneMinder Control Agent/".ZM_VERSION );

	#print( "http://$address/$cmd\n" );
	my $req = HTTP::Request->new( POST=>"http://$address/$cmd");
        $req->content($parms);
	my $res = $ua->request($req);

	if ( $res->is_success )
	{
		$result = !undef;
	}
	else
	{
		Error( "Error check failed: '".$res->status_line()."'\n" );
	}

	return( $result );
}

# moves the camera in any direction
sub move
{
	my $dir = shift;
	my $desc = shift;
	Debug( "Move $desc\n" );
	my $parms = "PanSingleMoveDegree=1&TiltSingleMoveDegree=1&PanTiltSingleMove=$dir";
	sendCmd( PANTILT, $parms );
}

sub presetClear
{
	my $preset = shift || 1;
	Debug( "Clear Preset $preset\n" );
	my $parms = "ClearPosition=$preset";
	sendCmd( PANTILT, $parms );
}

sub presetSet
{
        #TODO: need to figure out horizontal and vertical position
	my $preset = shift || 1;
	Debug( "Set Preset $preset\n" );
	my $parms = "PanTiltHorizontal=xx&PanTiltVertical=xx&SetName=$preset&SetPosition=$preset";
	sendCmd( PANTILT, $parms );
}

sub presetGoto
{
	my $preset = shift || 1;
	Debug( "Goto Preset $preset\n" );
	my $parms = "PanTiltPresetPositionMove=$preset";
	sendCmd( PANTILT, $parms );
}

sub presetHome
{
	Debug( "Home Preset\n" );
	my $parms = "PanTiltSingleMove=4";
	sendCmd( PANTILT, $parms );
}

#
# call the appropriate methods from the control command
#

if ( $command eq "move_rel_upleft" )
{
	move(0, "up left");
}
elsif ( $command eq "move_rel_up" )
{
	move(1, "up");
}
elsif ( $command eq "move_rel_upright" )
{
	move(2, "up right");
}
elsif ( $command eq "move_rel_left" )
{
	move(3, "left");
}
elsif ( $command eq "move_rel_right" )
{
	move(5, "right");
}
elsif ( $command eq "move_rel_downleft" )
{
	move(6, "down left");
}
elsif ( $command eq "move_rel_down" )
{
	move(7, "down");
}
elsif ( $command eq "move_rel_downright" )
{
	move(8, "down right");
}
elsif ( $command eq "preset_home" )
{
	presetHome();
}
elsif ( $command eq "preset_set" )
{
	presetSet( $preset );
}
elsif ( $command eq "preset_goto" )
{
	presetGoto( $preset );
}
else
{
	Error( "Can't handle command $command\n" );
}
